unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, SvcMgr, Dialogs,
  IdBaseComponent, IdComponent, IdTCPServer, NTSecurityU;

type
  TMyServiceName = class(TService)
    IdTCPServer1: TIdTCPServer;
    procedure IdTCPServer1Connect(AThread: TIdPeerThread);
    procedure IdTCPServer1Disconnect(AThread: TIdPeerThread);
    procedure ServiceCreate(Sender: TObject);
    procedure ServiceStart(Sender: TService; var Started: Boolean);
    procedure ServiceStop(Sender: TService; var Stopped: Boolean);
    procedure ServiceExecute(Sender: TService);
    procedure IdTCPServer1BeforeCommandHandler(ASender: TIdTCPServer;
      const AData: String; AThread: TIdPeerThread);
    procedure IdTCPServer1Exception(AThread: TIdPeerThread;
      AException: Exception);
    procedure IdTCPServer1ListenException(AThread: TIdListenerThread;
      AException: Exception);
    procedure IdTCPServer1Status(ASender: TObject;
      const AStatus: TIdStatus; const AStatusText: String);
    procedure IdTCPServer1NoCommandHandler(ASender: TIdTCPServer;
      const AData: String; AThread: TIdPeerThread);
  private
    { Private declarations }
  public
    function GetServiceController: TServiceController; override;
    { Public declarations }
  end;

var
  MyServiceName: TMyServiceName;

implementation

{$R *.DFM}

procedure log(s:string);
var f:textfile;
begin
  assignfile(f,'c:\logfile.log');
  try
    append(f);
  except
    rewrite(f);
  end;
  writeln(f,s);
  closefile(f);
end;

procedure ServiceController(CtrlCode: DWord); stdcall;
begin
  MyServiceName.Controller(CtrlCode);
end;

function TMyServiceName.GetServiceController: TServiceController;
begin
  Result := ServiceController;
end;

procedure TMyServiceName.IdTCPServer1Connect(AThread: TIdPeerThread);
begin
// client connected. save the connection if multi connection will be supported or needed
  log('client connected'); 
end;

procedure TMyServiceName.IdTCPServer1Disconnect(AThread: TIdPeerThread);
begin
  // remove client from saved list if multi connection will be supported or needed
  log('client disconnected');
end;

procedure TMyServiceName.ServiceCreate(Sender: TObject);
begin
  log('service created');
  log('is service runnig with admin privileges? '+booltostr(IsAdmin,true));
end;

procedure TMyServiceName.ServiceStart(Sender: TService; var Started: Boolean);
begin
  log('');
  log('---');
  log('');
  log('starting');
  IdTCPServer1.Active:=true;
end;

procedure TMyServiceName.ServiceStop(Sender: TService; var Stopped: Boolean);
begin
  log('stopping');
  IdTCPServer1.Active:=false;
end;

procedure TMyServiceName.ServiceExecute(Sender: TService);
begin
  while not terminated do
  begin
    sleep(500);
    ServiceThread.ProcessRequests(false);
  end;
end;

function SetUserRWEDAccessToFileOrFolderAndSubobjects( aUser : string; aFileOrFolder : String) : BOOLEAN; stdcall; 
Var 
  SID : Pointer; 
begin 
  SID := GetAccountSID ( aUser ); 
  try 
    Result := SetFileObjectAndSubobjectsAccessRights(aFileOrFolder, 
                                                     SID, 
                                                     GENERIC_READ + GENERIC_WRITE + GENERIC_EXECUTE + _DELETE 
                                                     ); 
  finally 
    FreeSid(SID); 
  end; 
end; 

function UnSetUserRWEDAccessToFileOrFolderAndSubobjects( aUser : string; aFileOrFolder : String) : BOOLEAN; stdcall;
Var
  SID : Pointer;
begin
  SID := GetAccountSID ( aUser );
  try
    Result := SetFileObjectAndSubobjectsAccessRights(aFileOrFolder,
                                                     SID,
                                                     0
                                                     );
  finally
    FreeSid(SID);
  end;
end;

procedure TMyServiceName.IdTCPServer1BeforeCommandHandler(ASender: TIdTCPServer;
  const AData: String; AThread: TIdPeerThread);
var s:TStringList; 
begin
 // incomming connection
   s:=TStringList.Create;
   s.CommaText:=AData;
   log('received: '+AData);
   if s[0]='adduser' then
   begin
     log('command: adduser');
     log('username: '+s[1]);
     log('path: '+s[2]);
     try
       log('setting was successfull? '+booltostr(SetUserRWEDAccessToFileOrFolderAndSubobjects(s[1], s[2]),true));
     except
       on e:exception do
         log('error while adding user: '+e.message);
     end;
     log('-');
   end               else
   if s[0]='remuser' then
   begin
     log('command: remuser');
     log('username: '+s[1]);
     log('path: '+s[2]);
     try
       log('unsetting was successfull? '+booltostr(UnSetUserRWEDAccessToFileOrFolderAndSubobjects(s[1], s[2]),true));
     except
       on e:exception do
         log('error while adding user: '+e.message);
     end;
     log('-');
   end               else
     log('unknown command');
   s.Free;
end;

procedure TMyServiceName.IdTCPServer1Exception(AThread: TIdPeerThread;
  AException: Exception);
begin
  log('exception: '+AException.Message);
end;

procedure TMyServiceName.IdTCPServer1ListenException(AThread: TIdListenerThread;
  AException: Exception);
begin
  log('listen exception: '+AException.Message);
end;

procedure TMyServiceName.IdTCPServer1Status(ASender: TObject;
  const AStatus: TIdStatus; const AStatusText: String);
begin
  log('status: '+AStatusText);
end;

procedure TMyServiceName.IdTCPServer1NoCommandHandler(
  ASender: TIdTCPServer; const AData: String; AThread: TIdPeerThread);
begin
// it's here only because if this event is not assigned AND there is no command handlers, an exception will be thrown
// you should implement command handlers, it will look nicer ;)
end;

end.
